@@ -33,8 +33,6 @@
 #define WNCK_I_KNOW_THIS_IS_UNSTABLE
 #include <libwnck/libwnck.h>
 
-#include <libmate-desktop/mate-aboutdialog.h>
-
 #include "wncklet.h"
 #include "showdesktop.h"
 
@@ -533,10 +531,11 @@ static void display_about_dialog(GtkAction* action, ShowDesktopData* sdd)
 	/* Translator credits */
 	//const char* translator_credits = _("translator-credits");
 	char copyright[] = \
+		"Copyright \xc2\xa9 2012-2016 MATE developers\n"
 		"Copyright \xc2\xa9 2011 Perberos\n"
 		"Copyright \xc2\xa9 2002 Red Hat, Inc.";
 
-	mate_show_about_dialog(GTK_WINDOW(sdd->applet),
+	gtk_show_about_dialog(GTK_WINDOW(sdd->applet),
 		"program-name", _("Show Desktop Button"),
 		"authors", authors,
 		"comments", _("This button lets you hide all windows and show the desktop."),
