@@ -34,13 +34,6 @@
 
 #include <string.h>
 
-#if defined(HAVE_CRT_EXTERNS_H) && defined(HAVE__NSGETENVIRON)
-#include <crt_externs.h> /* for _NSGetEnviron */
-#define environ (*_NSGetEnviron())
-#else
-extern char **environ;
-#endif
-
 static int            screens     = 0;
 static int           *monitors    = NULL;
 static GdkRectangle **geometries  = NULL;
@@ -728,38 +721,3 @@ panel_multiscreen_is_at_visible_extreme (GdkScreen *screen,
 	}
 }
 
-char **
-panel_make_environment_for_screen (GdkScreen  *screen,
-				   char      **envp)
-{
-	char **retval = NULL;
-	char  *display_name;
-	int    display_index = -1;
-	int    i, env_len;
-
-	g_return_val_if_fail (GDK_IS_SCREEN (screen), NULL);
-
-	if (envp == NULL)
-		envp = environ;
-
-	for (env_len = 0; envp[env_len]; env_len++)
-		if (strncmp (envp[env_len], "DISPLAY", strlen ("DISPLAY")) == 0)
-			display_index = env_len;
-
-	retval = g_new (char *, env_len + 1);
-	retval[env_len] = NULL;
-
-	display_name = gdk_screen_make_display_name (screen);
-
-	for (i = 0; i < env_len; i++)
-		if (i == display_index)
-			retval[i] = g_strconcat ("DISPLAY=", display_name, NULL);
-		else
-			retval[i] = g_strdup (envp[i]);
-
-	g_assert (i == env_len);
-
-	g_free (display_name);
-
-	return retval;
-}
